function [sol,stats] = odetest_61(problem,solver,atol,rtol,plotsel,opt) 
% Solve a problem in the testset format using the MATLAB ODE solvers  
% This version was tested with MATLAB 6.1  
%   
% [sol,stats] = odetest_61(problem,solver,atol,rtol,plotsel,opt) 
%   
% INPUT  
% problem = string with the name of the IVP TEST SET problem  
% ODE problems  
%     Hires, Pollu, Ringmod, Medakzo, Emep, Plei, vdpol,  
%     Rober, Orego, E5  
% DAE problems  
%     Chemakzo, Andrews, Transamp, Pump, Tba, Caraxis  
%     Fekete, Crank, Water  
%  
% solver  = string with the calling name of the MATLAB solver  
% atol = absolute tolerance  
% rtol = relative tolerance  
% plotsel = index of component to be plotted (optional)   
% opt = solver parameters set with ODESET (optional)   
%     
% OUTPUT  
% sol.x  : mesh point of the approximate solution  
% sol.y  : solution computed at sol.x  
% sol.solver : solver used  
% sol.scd : significant digit of the approximate solution  
% sol.mescd : mixed significant digit of the approximate solution  
% sol.ierr  : error flag (0 = successfull run, 1 = error) 
%  
% stats.nsteps : number of successfull steps  
% stats.nfailed : number of failed steps  
% stats.nfevals : number of function evaluation  
% stats.npds : number of partial derivative  
% stats.ndecomps : number of LU decomposition  
% stats.nsolves : number of solution of linear systems  
%   
%---------------------------------------------------------------------  
%     This file is part of the Test Set for IVP solvers  
%     http://www.dm.uniba.it/~testset/  
%  
%  
%     DISCLAIMER: see  
%     http://www.dm.uniba.it/~testset/disclaimer.php 
%  
%     The most recent version of this source file can be found at  
%     http://www.dm.uniba.it/~testset/matlab/odetest_61.m  
%  
%     This is revision 
%     $Id: odetest_61.m,v 1.2 2008/02/08 11:09:39 testset Exp $ 
%-----------------------------------------------------------------------  
  
if nargin < 4  
  error('Not enough input arguments')  
end    
sol = [];  
stats = [];  
  
% Extract problem data  
mproblem = minterface(problem);  
data = feval(mproblem,'Prob');  
  
% Adjust tolerances  
[atolv,rtolv] = feval(mproblem,'Tolerance',data.neqn,atol,rtol);  
if any(atolv ~= atol)  
  warning('Absolute tolerance have been changed');  
end  
if any(rtolv ~= rtol)   
  error('Relative tolerance must be a scalar');     
end  
  
% Prepare solver input  
if length(data.t) > 2   
  warning(sprintf('%s will not restart at the intermediate points: t = %s',...  
          upper(solver),sprintf('%g ',data.t(2:end-1))));  
end  
tspan = [data.t(1), data.t(end)];  
  
% ODE function  
ode = @odefun;  
  
switch data.type  
    
 case 'ODE'  
  y0 = feval(mproblem,'Init',data.neqn,data.t(1));  
  M0 = [];  % for odeset  
  yp0 = []; % for odeset  
    
 case 'DAE'  
  if any(data.ind > 1)  
    error(sprintf('Index %i reported. %s cannot solve DAEs of index > 1.', ...  
          max(data.ind),upper(solver)));  
  end  
  [y0,yp0] = feval(mproblem,'Init',data.neqn,data.t(1));  
  t0 = data.t(1);    
  % Mass matrix (constant)  
  m = data.neqn;  
  mlmas = data.mlmas;  
  mumas = data.mumas;  
  if mlmas == m  
    % Full storage  
    M0 = feval(mproblem,'Meval',m,m,t0,y0,yp0,[],[]);  
  else    
    % Band storage  
    ldim = mlmas + mumas + 1;  
    M0 = feval(mproblem,'Meval',ldim,m,t0,y0,yp0,[],[]);  
    M0 = spdiags(M0',[mumas:-1:-mlmas],m,m);  
  end        
    
 case 'IDE'  
  error('For IDEs, use IDETEST/ODE15i (MATLAB 7.0 and later).');  
    
end   
  
% Check for analytical Jacobian   
jac = [];  
if ~data.numjac  
  jac = @jacfun;  
end  
  
% Set solver options  
if nargin < 6 | ~isstruct(opt) 
  opt = [];  
end 
opt = odeset(opt,'Stats','on'); 
opt = odeset(opt,'Reltol',rtol,'Abstol',atolv);  
opt = odeset(opt,'Jacobian',jac);               % could be empty  
opt = odeset(opt,'Mass',M0,'InitialSlope',yp0); % could be empty  
if nargin < 5 
  plotsel = []; 
end 
if ~isempty(plotsel) 
  opt = odeset(opt,'OutputFcn',@odeplot,'OutputSel',plotsel);  
end  
  
% Print test info  
fprintf('%s \n\n',data.fullnm);  
fprintf('Solver: %s \n\n', upper(solver));  
fprintf('%e absolute tolerance \n', atol);  
fprintf('%e relative tolerance \n\n', rtol);  
  
% Define and evaluate the calling sequence string  
call_solver_str = [' sol = ',solver,...  
      '(ode,tspan,y0,opt,data.neqn,mproblem,data.mljac,data.mujac);'];  
warning off  
S = evalc(call_solver_str);  
warning on  
if sol.x(end) < data.t(end)  
    sol.ierr=1;  
    return;  % did not finish successfully...  
end  
% Log the solver output  
fprintf('%s \n',S);  
  
% Extract stats  
S = strrep(S,'e+','++');  
S = strrep(S,'successful steps',' ');  
S = strrep(S,'failed attempts',' ');  
S = strrep(S,'function evaluations',' ');  
S = strrep(S,'partial derivatives',' ');  
S = strrep(S,'LU decompositions',' ');  
S = strrep(S,'solutions of linear systems',' ');  
%S = regexprep(S,'[^0-9 \+ \. ]',' ');  
S = strrep(S,'. ',' ');  
S = strrep(S,'++','e+');  
st = sscanf(S,'%g');  
stats.nsteps   = st(end-5);  
stats.nfailed  = st(end-4);  
stats.nfevals  = st(end-3);  
stats.npds     = st(end-2);  
stats.ndecomps = st(end-1);  
stats.nsolves  = st(end);  
     
% Extract reference solution and compute actual errors  
sol.atol = atolv;  
sol.rtol = rtolv;  
  
if sol.x(end) == data.t(end)    % ML 6.5 solvers hit tend exactly  
      
  yref = feval(mproblem,'Solut',data.neqn,data.t(end));   
  yfinal = sol.y(:,end);  
  tolvec = (length(atolv) > 1);  
  [mescd,scd] = feval(mproblem,'Report',data.neqn, ...  
                      yref,yfinal,problem,double(tolvec),atolv,rtolv);  
          
  fprintf('\n');                  
  fprintf('mescd = %e \n', mescd);  
  fprintf('scd   = %e \n \n', scd);  
    
  sol.scd = max(scd);  
  sol.mescd = max(mescd); 
  sol.ierr  = 0; 
else 
  sol.ierr = 1;  
end  
% end odetest_61  
  
% ==========================================================================  
% Sub-functions  
%   
  
function f = odefun(t,y,m,mproblem,mljac,mujac)  
  f = feval(mproblem,'Feval',m,t,y,[],[],[]);  
% end odefun  
  
% --------------------------------------------------------------------------      
  
function J = jacfun(t,y,m,mproblem,mljac,mujac)  
  if mljac == m  % full Jacobian  
    J = feval(mproblem,'Jeval',m,m,t,y,[],[],[]);  
  else  % unpack sparse Jacobian  
    ldim = mljac + mujac + 1;  
    J = feval(mproblem,'Jeval',ldim,m,t,y,[],[],[]);  
    J = spdiags(J',[mujac:-1:-mljac],m,m);  
  end      
% end jacfun  
  
% --------------------------------------------------------------------------      
  
%EOF [odetest_61.m]  
